<?php
/*
Version: 1.1
Plugin Name: D&D Dice Roller
Description: A simple D&D dice roller widget that will let users roll a d20 as well as enter their Dungeons & Dragons skill modifier.  Clicking roll displays the modified result as well as the natural number rolled.
Author URI: http://dashfien.com
Author: David Dashifen Kees
*/

add_action("widgets_init", "dnd_dice_roller::load_widget");
add_action("wp_print_styles", "dnd_dice_roller::load_styles");
add_action("wp_enqueue_scripts", "dnd_dice_roller::load_scripts");
add_action('wp_ajax_nopriv_dnd_roll', 'dnd_dice_roller::roll');					// for unauthenticated visitors
add_action('wp_ajax_dnd_roll', 'dnd_dice_roller::roll');						// for authenticated visitors

class dnd_dice_roller extends WP_Widget {
	const version = 1.1;
	
	public static function load_widget() { register_widget("dnd_dice_roller"); }
	
	public static function load_scripts() {
		if(!defined("DND_PLUGIN_FOLDER")) {
			$folders = explode(DIRECTORY_SEPARATOR, dirname(__FILE__));
			while(($folder = array_pop($folders)) != "plugins") $path[] = $folder;
			define("DND_PLUGIN_FOLDER", WP_PLUGIN_URL . DIRECTORY_SEPARATOR . join(DIRECTORY_SEPARATOR, array_reverse($path)) . DIRECTORY_SEPARATOR);		
		}

		wp_enqueue_script("dnd_dice_roller_scripts", DND_PLUGIN_FOLDER . "dnd_dr_scripts.js", array("prototype"));
		wp_localize_script("dnd_dice_roller_scripts", 'dnd_dice_roller_ajax', array('url' => admin_url('admin-ajax.php')));
		wp_enqueue_script("prototype");
	}
	
	public static function load_styles() {
		if(!defined("DND_PLUGIN_FOLDER")) {
			$folders = explode(DIRECTORY_SEPARATOR, dirname(__FILE__));
			while(($folder = array_pop($folders)) != "plugins") $path[] = $folder;
			define("DND_PLUGIN_FOLDER", WP_PLUGIN_URL . DIRECTORY_SEPARATOR . join(DIRECTORY_SEPARATOR, array_reverse($path)) . DIRECTORY_SEPARATOR);		
		}

		wp_enqueue_style("dnd_dice_roller_styles", DND_PLUGIN_FOLDER . "dnd_dr_styles.css");
	}
		
	public static function roll() {
		extract($_POST);
		
		// the extraction above gives us $mod which we use to modify our roll.  if $mod != 0 we also show the 
		// natural roll in addition to the modified one.
		
		$natural = mt_rand(1, 20);
		$modified = $natural + (!is_numeric($mod) ? 0 : floor($mod));
		$format  = "<strong>Modified: </strong> %d<br><strong>Natural: </strong> %s";
		$results = $modified == $natural ? "$natural" : sprintf($format, $modified, $natural);
		if($natural == 20) $results .= "<em>&nbsp;Crit!</em>";
		exit($results);
	}
	

	/* WIDGET FUNCTIONALITY
	 * The following methods are the core of the WordPress widget functionality.  They provide the ability for our
	 * plugin to be hooked into the WordPress Appearance > Widgets list and create the on-screen display for it.
	 */
	
	public function __construct() {
		$widget_ops  = array("classname" => "dnd-dice-roller-widget", "description" => "A simple D&D dice roller widget that will let users roll a d20 as well as enter their Dungeons & Dragons skill modifier.");
		parent::__construct("dnd-dice-roller-widget", "D&amp;D Dice Roller", $widget_ops);
	}
	
	public function widget($args, $instance) {
		extract($instance);		// creates dnd_dice_roller_title, dnd_dice_roller_allow_mod
		extract($args);			// creates before_widget, before_title, after_title, after_widget
		
		$widget = "$before_widget $before_title $dnd_dice_roller_title $after_title";
		ob_start(); ?>
		
		<form id="dnd_dice_rolling_form">
		<p>
			<?php if($dnd_dice_roller_allow_mod) { ?>
				<label for="dnd_dice_roller_mod"><span>What is your</span> Modifier:</label>
				<input type="number" id="dnd_dice_roller_mod" name="dnd_dice_roller_mod" value="0">
			<?php } else { ?>
				<input type="hidden" id="dnd_dice_roller_mod" name="dnd_dice_roller_mod" value="0">
			<?php } ?>
		
			<input type="button" id="dnd_dice_roller_go" value="Roll d20">
		</p>
		</form>

		<div id="dnd_dice_roller_results"><strong>Result:</strong><p></p></div>
		<p id="dnd_dice_roller_link"><a href="http://www.awesomedice.com/wordpress-dice-roller">Awesome Dice</a></p>
		
		<?php $widget .= ob_get_clean() . $after_widget;		
		echo $widget;
	}

	public function update($new_instance, $old_instance) {
		$instance = $old_instance;
		$instance["dnd_dice_roller_title"] = strip_tags($new_instance["dnd_dice_roller_title"]);
		$instance["dnd_dice_roller_allow_mod"] = $new_instance["dnd_dice_roller_allow_mod"]==0 ? false : true;
		return $instance;
	}

	public function form($instance) {
		$defaults = array(
			"dnd_dice_roller_allow_mod" => true,
			"dnd_dice_roller_title" => "D&amp;D Dice Roller"
		);
		
		$instance = wp_parse_args((array) $instance, $defaults); ?>	
		
		<p>
			<label for="<?php echo $this->get_field_id("dnd_dice_roller_title"); ?>"><strong>Title:</strong></label>
			<input id="<?php echo $this->get_field_id("dnd_dice_roller_title"); ?>" name="<?php echo $this->get_field_name("dnd_dice_roller_title"); ?>" value="<?php echo $instance["dnd_dice_roller_title"]; ?>" size="25">
		</p>
		<p>
			<strong>Allow Modifiers:</strong><br>
			&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<label for="<?php echo $this->get_field_id("dnd_dice_roller_allow_mod_yes");?>"><input id="<?php echo $this->get_field_id("dnd_dice_roller_allow_mod_yes"); ?>" name="<?php echo $this->get_field_name("dnd_dice_roller_allow_mod"); ?>" value="1" type="radio" <?php echo $instance["dnd_dice_roller_allow_mod"] ? "checked" : ""?>> &ndash; Yes</label><br>
			&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<label for="<?php echo $this->get_field_id("dnd_dice_roller_allow_mod_no");?>"><input id="<?php echo $this->get_field_id("dnd_dice_roller_allow_mod_no"); ?>" name="<?php echo $this->get_field_name("dnd_dice_roller_allow_mod"); ?>" value="0" type="radio" <?php echo !$instance["dnd_dice_roller_allow_mod"] ? "checked" : ""?>> &ndash; No</label>
		<p>		
	<?php }


} ?>